@@ -34,7 +34,9 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <byteswap.h>
+#ifdef HAVE_PTHREAD_H
 #include <pthread.h>	// For pthread_atfork
+#endif
 #ifdef HAVE_SYSCALL_H
 #include <sys/syscall.h>
 #endif
@@ -153,15 +155,6 @@ static __thread struct cap_ng m =	{ 1,
 					CAPNG_NEW,
 					{0, 0} };
 
-
-/*
- * The pthread_atfork function is being made weak so that we can use it
- * if the program is linked with pthreads and not requiring it for
- * everything that uses libcap-ng.
- */
-extern int __attribute__((weak)) pthread_atfork(void (*prepare)(void),
-	void (*parent)(void), void (*child)(void));
-
 /*
  * Reset the state so that init gets called to erase everything
  */
@@ -173,8 +166,7 @@ static void deinit(void)
 static void init_lib(void) __attribute__ ((constructor));
 static void init_lib(void)
 {
-	if (pthread_atfork)
-		pthread_atfork(NULL, NULL, deinit);
+	pthread_atfork(NULL, NULL, deinit);
 }
 
 static void init(void)
@@ -777,10 +769,10 @@ capng_results_t capng_have_capabilities(capng_select_t set)
 				full = 1;
 			else
 				return CAPNG_PARTIAL;
-			if ((m.data.v3[1].effective & UPPER_MASK) == 0)
+			if ((m.data.v3[1].effective & UPPER_MASK) == 0 && !full)
 				empty = 1;
 			else if ((m.data.v3[1].effective & UPPER_MASK) ==
-							UPPER_MASK)
+						UPPER_MASK && !empty)
 				full = 1;
 			else
 				return CAPNG_PARTIAL;
@@ -811,6 +803,40 @@ capng_results_t capng_have_capabilities(capng_select_t set)
 	return CAPNG_PARTIAL;
 }
 
+// -1 - error, 0 - no caps, 1 partial caps, 2 full caps
+capng_results_t capng_have_permitted_capabilities(void)
+{
+	int empty = 0, full = 0;
+
+	// First, try to init with current set
+	if (m.state < CAPNG_INIT)
+		capng_get_caps_process();
+
+	// If we still don't have anything, error out
+	if (m.state < CAPNG_INIT)
+		return CAPNG_FAIL;
+
+	if (m.data.v3[0].permitted == 0)
+		empty = 1;
+	else if (m.data.v3[0].permitted == 0xFFFFFFFFU)
+		full = 1;
+	else
+		return CAPNG_PARTIAL;
+	if ((m.data.v3[1].permitted & UPPER_MASK) == 0 && !full)
+		empty = 1;
+	else if ((m.data.v3[1].permitted & UPPER_MASK) == UPPER_MASK && !empty)
+		full = 1;
+	else
+		return CAPNG_PARTIAL;
+
+	if (empty == 1 && full == 0)
+		return CAPNG_NONE;
+	else if (empty == 0 && full == 1)
+		return CAPNG_FULL;
+	
+	return CAPNG_PARTIAL;
+}
+
 static int check_effective(unsigned int capability, unsigned int idx)
 {
 	return MASK(capability) & m.data.v3[idx].effective ? 1 : 0;
